import { RootConfig, AppConfig } from 'noodl-types';
import { Compiler } from 'webpack';
import WebSocket from 'ws';
import chokidar from 'chokidar';
import express from 'express';
import * as t from './types';
import { SetRequired } from 'type-fest';
interface Options {
    config?: string;
    deviceType?: 'web' | 'android' | 'ios';
    env?: 'test' | 'stable';
    hostname?: string;
    locale?: string;
    staticPaths?: [
        route: string,
        request: (req: Parameters<express.RequestHandler>[0]) => void,
        response: (res: Parameters<express.RequestHandler>[1]) => void
    ][];
    serverDir?: string;
    serverPort?: number | string;
    version?: 'latest' | number;
    wssPort?: number | string;
}
declare class NoodlWebpackPlugin {
    #private;
    options: Required<Options>;
    static pluginName: string;
    constructor(options: SetRequired<Options, 'config'>);
    get config(): string;
    set config(config: string);
    get server(): express.Express;
    get watcher(): chokidar.FSWatcher;
    get wss(): WebSocket.Server;
    get serverUrl(): string;
    get wssUrl(): string;
    apply(compiler: Compiler): Promise<void>;
    getRoutes({ filename, group }: t.MetadataObject): string[];
    /**
     * @param { object } options
     * @param { express.Express } options.server
     * @param { WebSocket.Server } options.wss
     * @param { chokidar.FSWatcher } options.watch
     */
    listen({ server, wss, watch, }: {
        server: express.Express;
        wss: WebSocket.Server;
        watch: chokidar.FSWatcher;
    }): void;
    /**
     * @param { Partial<RootConfig> } configObject
     * Parses the config provided, returning a parsed object where placeholders
     * are replaced
     */
    parseConfig(rootConfig: RootConfig): RootConfig;
    parseConfig(appConfig: AppConfig, rootConfig: RootConfig): AppConfig;
    getConfigVersion(configObject?: RootConfig): any;
}
export = NoodlWebpackPlugin;
