"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _NoodlWebpackPlugin_cadlBaseUrl, _NoodlWebpackPlugin_other, _NoodlWebpackPlugin_assets, _NoodlWebpackPlugin_server, _NoodlWebpackPlugin_watch, _NoodlWebpackPlugin_wss, _NoodlWebpackPlugin_yml;
const u = __importStar(require("@jsmanifest/utils"));
const get_1 = __importDefault(require("lodash/get"));
const invariant_1 = __importDefault(require("invariant"));
const yaml_1 = require("yaml");
const ws_1 = __importDefault(require("ws"));
const chokidar_1 = __importDefault(require("chokidar"));
const express_1 = __importDefault(require("express"));
const globby_1 = __importDefault(require("globby"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const utils_1 = require("./utils");
const c = __importStar(require("./constants"));
const pluginName = 'noodl-webpack-plugin';
const tag = `[${u.cyan(pluginName)}]`;
const info = (s1, ...s2) => u.log(`${tag} ${s1}`, ...s2);
const getAbsPath = (...s) => path_1.default.resolve(path_1.default.join(process.cwd(), ...s));
class NoodlWebpackPlugin {
    constructor(options) {
        _NoodlWebpackPlugin_cadlBaseUrl.set(this, '');
        _NoodlWebpackPlugin_other.set(this, []);
        _NoodlWebpackPlugin_assets.set(this, []);
        _NoodlWebpackPlugin_server.set(this, void 0);
        _NoodlWebpackPlugin_watch.set(this, void 0);
        _NoodlWebpackPlugin_wss.set(this, void 0);
        _NoodlWebpackPlugin_yml.set(this, []);
        this.options = {};
        const { config, deviceType = c.DEFAULT_DEVICE_TYPE, env = c.DEFAULT_ENV, hostname = c.DEFAULT_HOSTNAME, locale = c.DEFAULT_LOCALE, serverDir = c.DEFAULT_SERVER_PATH, serverPort = c.DEFAULT_SERVER_PORT, staticPaths, wssPort = c.DEFAULT_WSS_PORT, version = c.DEFAULT_CONFIG_VERSION, } = options;
        invariant_1.default(!!config, `A config value is required`);
        this.options.config = config;
        this.options.deviceType = deviceType;
        this.options.env = env;
        this.options.hostname = hostname;
        this.options.locale = locale;
        this.options.serverDir = getAbsPath(serverDir);
        this.options.serverPort = serverPort;
        this.options.staticPaths = staticPaths || [];
        this.options.version = version;
        this.options.wssPort = wssPort;
        invariant_1.default(fs_extra_1.default.existsSync(this.options.serverDir), `The path ${this.options.serverDir} does not exist`);
    }
    get config() {
        return this.options.config;
    }
    set config(config) {
        this.options.config = config;
    }
    get server() {
        return __classPrivateFieldGet(this, _NoodlWebpackPlugin_server, "f");
    }
    get watcher() {
        return __classPrivateFieldGet(this, _NoodlWebpackPlugin_watch, "f");
    }
    get wss() {
        return __classPrivateFieldGet(this, _NoodlWebpackPlugin_wss, "f");
    }
    get serverUrl() {
        return `http://${this.options.hostname}:${this.options.serverPort}`;
    }
    get wssUrl() {
        return `ws://${this.options.hostname}:${this.options.wssPort}`;
    }
    async apply(compiler) {
        var _a, _b;
        u.newline();
        info(`Config: ${u.green(this.config)}`);
        info(`Serving files at: ${u.green(this.serverUrl)}`);
        info(`Path to server files: ${u.green(this.options.serverDir)}`);
        const { config, deviceType, env, hostname, locale, serverPort, serverDir, staticPaths, version, } = this.options;
        info(`Searching your directory for the config file...`);
        const localFiles = (await globby_1.default(path_1.default.join(serverDir, '**/*'), {
            onlyFiles: true,
        }));
        //
        let configRegEx = new RegExp(`(${this.config}|${this.config}.yml)`, 'i');
        let configFilePath = localFiles.find((filepath) => configRegEx.test(filepath));
        let configDoc;
        let configRemotelyFetched = false;
        let configYml = '';
        let configVersion = '';
        if (configFilePath) {
            try {
                info(`Found ${u.yellow(configFilePath)}`);
                configDoc = yaml_1.parseDocument(utils_1.loadFile(configFilePath));
            }
            catch (error) {
                throw error;
            }
        }
        else {
            info(`Config file not found. ` +
                `Downloading remotely and saving it to your dir...`);
            // Prompt first time config
            // Else use saved config as default
            configYml = (await utils_1.request(`https://public.aitmed.com/config/${config}.yml`));
            configRemotelyFetched = true;
            info(`Received config in (YML)`);
            configDoc = yaml_1.parseDocument(configYml);
            try {
                const configFileName = `${config}.yml`;
                await fs_extra_1.default.writeFile(path_1.default.join(serverDir, configFileName), yaml_1.stringify(configDoc), 'utf8');
                info(`Created and saved ${u.yellow(configFileName)} to your dir`);
            }
            catch (error) {
                throw new Error(error);
            }
        }
        info(`Loading the config using ${u.yellow(deviceType)} with version ` +
            `${u.yellow(String(version))} in ${u.yellow(String(env))} ` +
            `environment (You can change these settings by passing them into the plugin's options)`);
        let rootConfigObject = configDoc.toJSON();
        let appConfig;
        let appConfigYml = '';
        let appConfigUrl = '';
        configVersion = this.getConfigVersion(rootConfigObject);
        info(`Selected config version: ${u.yellow(configVersion)}${version === 'latest' ? ` (latest)` : ''}`);
        const parsedRootConfig = this.parseConfig(rootConfigObject);
        appConfigUrl = `${parsedRootConfig.cadlBaseUrl}${parsedRootConfig.cadlMain}`;
        info(`Root config was parsed`);
        __classPrivateFieldSet(this, _NoodlWebpackPlugin_cadlBaseUrl, parsedRootConfig.cadlBaseUrl, "f");
        info(`apiHost: ${u.yellow(parsedRootConfig.apiHost)}`);
        info(`appApiHost: ${u.yellow(parsedRootConfig.appApiHost)}`);
        info(`webApiHost: ${u.yellow(parsedRootConfig.webApiHost)}`);
        info(`apiPort: ${u.yellow(parsedRootConfig.apiPort)}`);
        info(`cadlBaseUrl: ${u.yellow(__classPrivateFieldGet(this, _NoodlWebpackPlugin_cadlBaseUrl, "f"))}`);
        if (parsedRootConfig.myBaseUrl) {
            info(`myBaseUrl: ${u.yellow(parsedRootConfig.myBaseUrl)}`);
        }
        if (parsedRootConfig.timestamp) {
            info(`timestamp: ${u.yellow(parsedRootConfig.timestamp)}`);
        }
        if (configRemotelyFetched) {
            info(`Retrieving app config from ${u.yellow(appConfigUrl)}`);
            appConfigYml = (await utils_1.request(appConfigUrl));
        }
        else {
            const appConfigFilePath = localFiles.find((filepath) => filepath.endsWith(parsedRootConfig.cadlMain));
            console.log('parsedRootConfig', parsedRootConfig);
            if (appConfigFilePath) {
                info(`Loading app config...`);
                appConfigYml = utils_1.loadFile(appConfigFilePath);
            }
            else {
                info(`Missing app config file from the dir. Fetching remotely now...`);
                appConfigYml = (await utils_1.request(appConfigUrl));
            }
        }
        if (appConfigYml)
            info(`App config was retrieved as YAML`);
        else
            throw new Error(`Could not load the app config YAML`);
        appConfig = yaml_1.parse(appConfigYml);
        if (appConfig) {
            appConfig = this.parseConfig(appConfig, rootConfigObject);
            info(`App config was parsed`);
            info(`assetsUrl: ${u.yellow(appConfig.assetsUrl)}`);
            info(`baseUrl: ${u.yellow(appConfig.baseUrl)}`);
            info(`startPage: ${u.yellow(appConfig.startPage)}`);
            info(`No. preload pages: ${u.yellow(((_a = appConfig.preload) === null || _a === void 0 ? void 0 : _a.length) || 0)}`);
            info(`No. pages: ${u.yellow(((_b = appConfig.page) === null || _b === void 0 ? void 0 : _b.length) || 0)}`);
        }
        else {
            throw new Error(`Could not load the app config object`);
        }
        const preloadPages = appConfig.preload;
        const pages = appConfig.page;
        preloadPages.length &&
            info(`${u.yellow(String(preloadPages.length))} preload pages being loaded`);
        pages.length && info(`Total pages: ${u.yellow(String(pages.length))}`);
        /**
         * 1. Scan all files
         * 2. Separate config files from non-config files
         */
        if (localFiles.length) {
            info(`Loading ${u.yellow(String(localFiles.length))} files mentioned in ${u.green(`${this.config}.yml`)}`);
        }
        else {
            info(`No local files found in ${u.yellow(serverDir)}`);
        }
        const missingPages = [];
        for (const filepath of localFiles) {
            if (!(filepath === null || filepath === void 0 ? void 0 : filepath.includes('.')))
                continue;
            if (/\/assets?/i.test(filepath)) {
                __classPrivateFieldGet(this, _NoodlWebpackPlugin_assets, "f").push(utils_1.getMetadataObject(filepath));
            }
            else if (utils_1.isYml(filepath)) {
                __classPrivateFieldGet(this, _NoodlWebpackPlugin_yml, "f").push(utils_1.getMetadataObject(filepath, { config: this.config }));
            }
            else {
                __classPrivateFieldGet(this, _NoodlWebpackPlugin_other, "f").push(filepath);
            }
        }
        this.listen({
            server: (__classPrivateFieldSet(this, _NoodlWebpackPlugin_server, express_1.default(), "f")),
            wss: (__classPrivateFieldSet(this, _NoodlWebpackPlugin_wss, new ws_1.default.Server({
                host: this.options.hostname,
                port: Number(this.options.wssPort),
            }), "f")),
            watch: (__classPrivateFieldSet(this, _NoodlWebpackPlugin_watch, chokidar_1.default.watch(path_1.default.join(serverDir, '**/*'), {
                cwd: process.cwd(),
                ignoreInitial: true,
            }), "f")),
        });
    }
    getRoutes({ filename, group }) {
        if (group === 'page') {
            let route = utils_1.ensureBeginsWithSingleSlash(filename);
            utils_1.isYml(route) && (route = route.replace('.yml', ''));
            return [route, `${route}.yml`, `${route}_en.yml`];
        }
        else {
            return [filename, `/assets/${filename.replace('/', '')}`];
        }
    }
    /**
     * @param { object } options
     * @param { express.Express } options.server
     * @param { WebSocket.Server } options.wss
     * @param { chokidar.FSWatcher } options.watch
     */
    listen({ server, wss, watch, }) {
        const comet = '\u2604\uFE0F';
        const croissant = '\uD83D\uDE48';
        const tags = {
            ws: u.bold(u.cyan('ws')),
            host: u.yellow(this.options.hostname),
            port: u.yellow(String(this.options.serverPort)),
            slash: u.white('://'),
            config: u.yellow(this.options.config),
            watch: u.cyan(`watch`),
        };
        const registerRoute = (type) => {
            const register = (obj) => {
                let filename = utils_1.ensureBeginsWithSingleSlash(obj.filename);
                let routes;
                if (type === 'asset') {
                    //
                }
                else {
                    filename = utils_1.ensurePageName(filename);
                    obj.group = 'page';
                    const route = server.get((routes = this.getRoutes(obj)), (req, res, next) => {
                        res.send(utils_1.loadFile(obj.filepath));
                        next();
                    });
                }
            };
            return register;
        };
        server.use(express_1.default.static(this.options.serverDir));
        for (const obj of this.options.staticPaths) {
            server.get(this.getRoutes(utils_1.getMetadataObject(obj[0])), (req, res, next) => {
                var _a, _b;
                (_a = obj[1]) === null || _a === void 0 ? void 0 : _a.call(obj, req);
                (_b = obj[2]) === null || _b === void 0 ? void 0 : _b.call(obj, res);
                return next();
            });
        }
        const registerAssetRoute = registerRoute('asset');
        const registerPageRoute = registerRoute('page');
        __classPrivateFieldGet(this, _NoodlWebpackPlugin_assets, "f").forEach(registerAssetRoute);
        __classPrivateFieldGet(this, _NoodlWebpackPlugin_yml, "f").forEach(registerPageRoute);
        server.listen(this.options.serverPort, () => {
            wss
                .on('listening', () => { })
                .on('connection', (ws) => {
                ws.on('message', (message) => {
                    const data = (u.isStr(message) ? JSON.parse(message) : message);
                    info('Received: ', data);
                    if (data.from === 'webext') {
                        switch (data.type) {
                            case 'DOM_LOADED':
                            default:
                                break;
                        }
                    }
                    else {
                        if (data.type === 'track') {
                            //
                        }
                    }
                });
            })
                // .on('headers', (headers, { headers: headersObject, socket, url }) => {
                // 	info(`Headers string`, headers)
                // 	info(`Headers string`, headersObject)
                // 	info(`Socket info`, {
                // 		...pick(socket, [
                // 			'localAddress',
                // 			'localPort',
                // 			'remoteAddress',
                // 			'remoteFamily',
                // 			'remotePort',
                // 		] as (keyof typeof socket)[]),
                // 		address: socket.address(),
                // 	})
                // })
                .on('close', () => info(u.white(`WS has closed`)))
                .on('error', (err) => info(u.red(`[${err.name}] ${err.message}`)));
            function sendMessage(msg) {
                return new Promise((resolve, reject) => {
                    wss.clients.forEach((client) => {
                        client.send(JSON.stringify(msg, null, 2), (err) => {
                            if (err)
                                reject(err);
                            else
                                resolve(undefined);
                        });
                    });
                });
            }
            function getFileName(path) {
                path.includes('/') && (path = path.substring(path.lastIndexOf('/') + 1));
                return utils_1.ensurePageName(path);
            }
            function onWatchEvent(type, fn) {
                async function onEvent(path) {
                    const stats = await fs_extra_1.default.stat(path);
                    const args = {
                        isFile: stats.isFile(),
                        isFolder: stats.isDirectory(),
                        name: utils_1.ensurePageName(getFileName(path)),
                        path,
                        // stats,
                    };
                    sendMessage(Object.assign({ type }, args));
                    fn(args);
                }
                return onEvent;
            }
            watch
                .on('ready', () => {
                info(`Watching for file changes at ${u.green(this.options.serverDir)}`);
                sendMessage({ type: c.WATCHING_FILES });
            })
                .on('change', onWatchEvent(c.FILE_CHANGED, (args) => info(`File changed`, u.magenta(path_1.default.resolve(args.path)))))
                .on('add', onWatchEvent(c.FILE_ADDED, (args) => {
                var _a;
                info(`File added`, u.magenta(path_1.default.resolve(args.path)));
                if (utils_1.isYml(args.path)) {
                    let filepath = path_1.default.resolve(args.path);
                    let filename = args.path;
                    utils_1.isYml(filename) && (filename = filename.replace('.yml', ''));
                    if (((_a = filename.split('/')) === null || _a === void 0 ? void 0 : _a.length) > 2) {
                        filename = filename.substring(filename.lastIndexOf('/'));
                    }
                    info(u.white(`Added new route: ${u.magenta(filename)}`));
                    server === null || server === void 0 ? void 0 : server.get(this.getRoutes({
                        ext: 'yml',
                        filepath,
                        filename,
                        group: 'page',
                    }), (req, res) => res.send(utils_1.loadFile(filepath)));
                }
            }))
                .on('addDir', onWatchEvent(c.FOLDER_ADDED, (args) => info(`Folder added`, u.magenta(path_1.default.resolve(args.path)))))
                .on('unlink', onWatchEvent(c.FILE_REMOVED, (args) => info(`File was removed`, u.magenta(path_1.default.resolve(args.path)))))
                .on('unlinkDir', onWatchEvent(c.FOLDER_REMOVED, (args) => info(`Folder was removed`, u.magenta(path_1.default.resolve(args.path)))))
                .on('error', (err) => {
                info(`Error`, err);
                sendMessage({ type: c.WATCH_ERROR, error: err });
            });
        });
    }
    parseConfig(configObject1, configObject2) {
        // Root config
        if (u.isUnd(configObject2)) {
            configObject1 = configObject1;
            return JSON.parse(utils_1.createPlaceholderReplacers([
                ['${cadlBaseUrl}', __classPrivateFieldGet(this, _NoodlWebpackPlugin_cadlBaseUrl, "f")],
                ['${cadlVersion}', this.getConfigVersion(configObject1)],
            ], 'g')(JSON.stringify(Object.assign(Object.assign({}, configObject1), { webApiHost: configObject1.webApiHost === 'apiHost'
                    ? configObject1.apiHost
                    : configObject1.webApiHost, appApiHost: configObject1.appApiHost === 'apiHost'
                    ? configObject1.apiHost
                    : configObject1.appApiHost, cadlBaseUrl: configObject1.cadlBaseUrl || c.DEFAULT_CONFIG_BASEE_URL, loadingLevel: u.isNum(configObject1.loadingLevel)
                    ? configObject1.loadingLevel
                    : 1, debug: configObject1.debug || c.DEFAULT_CONFIG_DEBUG }))));
        }
        return JSON.parse(utils_1.createPlaceholderReplacers([
            ['${cadlBaseUrl}', __classPrivateFieldGet(this, _NoodlWebpackPlugin_cadlBaseUrl, "f")],
            ['${cadlVersion}', this.getConfigVersion(configObject2)],
        ], 'g')(u.isStr(configObject1) ? configObject1 : JSON.stringify(configObject1)));
    }
    getConfigVersion(configObject) {
        if (this.options.version === 'latest') {
            return get_1.default(configObject || {}, `${this.options.deviceType || 'web'}.cadlVersion.${this.options.env || 'test'}`);
        }
        return this.options.version;
    }
}
_NoodlWebpackPlugin_cadlBaseUrl = new WeakMap(), _NoodlWebpackPlugin_other = new WeakMap(), _NoodlWebpackPlugin_assets = new WeakMap(), _NoodlWebpackPlugin_server = new WeakMap(), _NoodlWebpackPlugin_watch = new WeakMap(), _NoodlWebpackPlugin_wss = new WeakMap(), _NoodlWebpackPlugin_yml = new WeakMap();
NoodlWebpackPlugin.pluginName = pluginName;
module.exports = NoodlWebpackPlugin;
//# sourceMappingURL=noodl-webpack-plugin.js.map