"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.request = exports.replaceVersionPlaceholder = exports.replaceTildePlaceholder = exports.replaceDesignSuffixPlaceholder = exports.replaceBaseUrlPlaceholder = exports.getMetadataObject = exports.loadFile = exports.isPageDoc = exports.isYml = exports.isVid = exports.isImg = exports.getExt = exports.ensurePageName = exports.ensureBeginsWithSingleSlash = exports.createPlaceholderReplacer = exports.createPlaceholderReplacers = void 0;
const u = __importStar(require("@jsmanifest/utils"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const https_1 = __importDefault(require("https"));
const yaml_1 = require("yaml");
function createPlaceholderReplacers(replacers, flags = 'g') {
    const regexReplacers = replacers.map(([placeholder, placeholderValue], i) => [
        new RegExp(placeholder.includes('$')
            ? placeholder.replace('$', '\\$')
            : placeholder, flags),
        placeholderValue,
    ]);
    function replace(str) {
        let data = u.isStr(str) ? str : JSON.stringify(str);
        for (const [regex, placeholderValue] of regexReplacers) {
            data = regex.test(data) ? data.replace(regex, placeholderValue) : data;
        }
        return data;
    }
    return replace;
}
exports.createPlaceholderReplacers = createPlaceholderReplacers;
function createPlaceholderReplacer(placeholders, flags) {
    const regexp = new RegExp(u.reduce(u.array(placeholders), (str, placeholder) => str +
        (!str
            ? placeholder.replace('$', '\\$')
            : `|${placeholder.replace('$', '\\$')}`), ''), flags);
    function replace(str, value) {
        if (u.isStr(str)) {
            return str.replace(regexp, String(value));
        }
        else if (u.isObj(str)) {
            const stringified = JSON.stringify(str).replace(regexp, String(value));
            return JSON.parse(stringified);
        }
        return '';
    }
    return replace;
}
exports.createPlaceholderReplacer = createPlaceholderReplacer;
function ensureBeginsWithSingleSlash(s) {
    if (!s.startsWith('/'))
        s = '/' + s;
    if (s.startsWith('//'))
        s = s.replace('//', '/');
    return s;
}
exports.ensureBeginsWithSingleSlash = ensureBeginsWithSingleSlash;
function ensurePageName(s) {
    if (isYml(s))
        s = s.substring(0, s.lastIndexOf('.yml'));
    if (s.startsWith('/'))
        s = s.replace('/', '');
    return s;
}
exports.ensurePageName = ensurePageName;
function getExt(str) {
    return str.includes('.') ? str.substring(str.lastIndexOf('.') + 1) : '';
}
exports.getExt = getExt;
function isImg(s) {
    return /([a-z\-_0-9/:.]*\.(jpg|jpeg|png|gif|bmp|tif))/i.test(s);
}
exports.isImg = isImg;
function isVid(s) {
    return /([a-z\-_0-9/:.]*\.(mp4|avi|wmv))/i.test(s);
}
exports.isVid = isVid;
function isYml(s) {
    return s.endsWith('.yml');
}
exports.isYml = isYml;
function isPageDoc(doc) {
    if (doc &&
        yaml_1.isDocument(doc) &&
        yaml_1.isMap(doc.contents) &&
        doc.contents.items.length === 1) {
        const node = doc.contents.items[0].key;
        if (yaml_1.isScalar(node) && typeof node.value === 'string') {
            return node.value[0] === node.value[0].toUpperCase();
        }
    }
    return false;
}
exports.isPageDoc = isPageDoc;
function loadFile(filepath) {
    return fs_extra_1.default.readFileSync(filepath, 'utf8');
}
exports.loadFile = loadFile;
exports.getMetadataObject = (function () {
    function getFilename(str = '') {
        return !str.includes('/') ? str : str.substring(str.lastIndexOf('/') + 1);
    }
    function getMetadataObject(value, { config } = {}) {
        const metadata = {};
        metadata.ext = getExt(value);
        metadata.filename = getFilename(value);
        metadata.filepath = value;
        if (isYml(value)) {
            if (metadata.filename.endsWith('.yml')) {
                metadata.filename = metadata.filename.substring(0, metadata.filename.lastIndexOf('.yml'));
            }
            metadata.group = metadata.filename === config ? 'config' : 'page';
        }
        else if (isImg(value)) {
            metadata.group = 'image';
        }
        else if (/(json|pdf)/i.test(value)) {
            metadata.group = 'document';
        }
        else if (isVid(value)) {
            metadata.group = 'video';
        }
        else if (value.endsWith('.html') || value.endsWith('.js')) {
            metadata.group = 'script';
        }
        return metadata;
    }
    return getMetadataObject;
})();
exports.replaceBaseUrlPlaceholder = createPlaceholderReplacer('${cadlBaseUrl}', 'g');
exports.replaceDesignSuffixPlaceholder = createPlaceholderReplacer('${designSuffix}', 'g');
exports.replaceTildePlaceholder = createPlaceholderReplacer('~/');
exports.replaceVersionPlaceholder = createPlaceholderReplacer('${cadlVersion}', 'g');
function request(url) {
    return new Promise((resolve, reject) => {
        const req = https_1.default.get(url);
        req.on('response', (res) => {
            let data = '';
            res
                .on('data', (chunk) => {
                if (Buffer.isBuffer(chunk))
                    data += chunk.toString('utf8');
                else if (u.isStr(chunk))
                    data += chunk;
            })
                .on('end', () => resolve(data))
                .on('error', reject);
        });
    });
}
exports.request = request;
//# sourceMappingURL=utils.js.map